package im.zego.chathouse.protocol;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.json.JSONArray;

import java.util.Collections;

import im.zego.chathouse.BuildConfig;
import im.zego.chathouse.model.HeartbeatRespInfo;
import im.zego.chathouse.model.NoInfo;
import im.zego.chathouse.model.RoomInfo;
import im.zego.chathouse.model.RoomListInfo;
import im.zego.enjoycommon.log.L;

/**
 * 房间API
 */
public class RoomAPI {

    private static final String TAG = "RoomAPI";

    /**
     * 拉取房间列表
     *
     * @param count 数量
     * @param beginTimestamp 开始时间
     * @param callback 回调
     */
    public static void getRoomList( int count, long beginTimestamp, @NotNull final IGetRoomListCallback callback) {
        JsonBuilder jsonBuilder = new AppJsonBuilder()
                .addParams("count", count)
                .addParams("begin_timestamp", beginTimestamp);


        String reqJson = jsonBuilder.build();
        APIBase.asyncPost("/chat_house/v1/get_room_list", reqJson, RoomListInfo.class, new IAsyncPostCallback<RoomListInfo>() {
            @Override
            public void onResponse(int retCode, @NotNull String message, RoomListInfo responseJsonBean) {
                if (callback != null) {
                    callback.onGetRoomList(retCode, responseJsonBean);
                }
            }
        });
    }

    /***
     * 创建房间
     *
     * @param subject 主题
     * @param coverImg 封面
     * @param callback 回调
     */
    public static void createRoom(String subject, String coverImg, @NotNull final ICreateRoomCallback callback)
    {
        L.i(TAG, "GrabLog, onCreateRoom()");

        JsonBuilder jsonBuilder = APIBase.builder()
                .addParams("subject", subject)
                .addParams("cover_img", coverImg)
                .addParams("max_mic",4);


        String reqJson = jsonBuilder.build();

        APIBase.asyncPost("/chat_house/v1/create_room",reqJson, RoomInfo.class, (errorCode, message, roomInfo) -> {
            L.i(TAG, "GrabLog, onCreateRoom(), retCode:%d, msg:%s, roomID:%s", errorCode, message, (roomInfo != null) ? roomInfo.getRoomId() : "");

            if(callback != null) {
               callback.onCreateRoom(errorCode, roomInfo);
           }
        });
    }

    /**
     * 进入房间
     *
     * @param roomID 房间ID
     * @param nickName 昵称
     * @param userIcon 用户头像
     * @param callback 回调
     */
    public static void loginRoom( String roomID, String nickName, String userIcon, @NotNull final ILoginRoomCallback callback) {
        L.i(TAG, "GrabLog, onLoginRoom(), roomID:%s", roomID);

        String reqJson = APIBase.builder()
                .addParams("room_id", roomID)
                .addParams("nick_name", nickName)
                .addParams("avatar", userIcon)
                .build();

        APIBase.asyncPost("/chat_house/v1/login_room", reqJson, RoomInfo.class, new IAsyncPostCallback<RoomInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, RoomInfo notifyRoomStateUpdateInfo) {
                L.i(TAG, "GrabLog, onLoginRoom(), retCode:%d, msg:%s", errorCode, message);

                if(callback != null) {
                    callback.onLoginRoom(errorCode, notifyRoomStateUpdateInfo);
                }
            }
        });
    }

    /***
     * 离开房间
     *
     * @param roomID 房间ID
     * @param callback 回调
     */
    public static void quitRoom(String roomID, @Nullable final ICommonResponseCallback callback)
    {

        String reqJson = APIBase.builder()
                .addParams("room_id", roomID)
                .build();

        APIBase.asyncPost("/chat_house/v1/quit_room", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retCode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retCode);
                }
            }
        });
    }

    /**
     * 关闭房间
     *
     * @param roomID 房间ID
     * @param callback 回调
     */
    public static void closeRoom(String roomID, @Nullable final ICommonResponseCallback callback) {


        String reqJson = APIBase.builder()
                .addParams("room_id", roomID)
                .build();

        APIBase.asyncPost("/chat_house/v1/close_room", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retCode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retCode);
                }
            }
        });
    }

    /**
     * 心跳
     * @param callback 回调
     */
    public static void heartBeat(String roomID, IHeartBeatCallback callback) {

        String reqJson = APIBase.builder()
                .addParams("room_id", roomID)
                .build();

        String api = "/chat_house/v1/heartbeat";
        APIBase.asyncPost(api, reqJson, HeartbeatRespInfo.class, new IAsyncPostCallback<HeartbeatRespInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, HeartbeatRespInfo responseJsonBean) {

                errorCodeNotify(null,api,errorCode);

                if (callback != null) {
                    callback.onHeartBeat(errorCode, responseJsonBean);
                }
            }
        });
    }

    /**
     * 请求错误码通知
     * @param tag 请求绑定的对象
     * @param api 请求的url
     * @param errorCode 请求响应的错误码
     */
    private static void errorCodeNotify(@Nullable  Object tag,String api,int errorCode){
        RequestErrorCodeManager.RequestErrorCodeListener listener = RequestErrorCodeManager.getInstance().getRequestErrorCodeListener();
        if(listener != null){
            listener.onRequestErrorCode(tag,api,errorCode);
        }
    }

}
