package im.zego.chathouse.room;

import static im.zego.chathouse.ErrorCode.ERROR_FAIL_NETWORK;
import static im.zego.chathouse.ErrorCode.ERROR_ROOM_FULL;
import static im.zego.chathouse.ErrorCode.ERROR_ROOM_NOT_EXIST;
import static im.zego.chathouse.ErrorCode.ERROR_UID_HAS_CREATED_ROOM;

import android.app.Application;
import android.os.SystemClock;

import im.zego.chathouse.R;
import im.zego.chathouse.room.callback.ILoginServiceCallback;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.RoomInfo;
import im.zego.chathouse.protocol.ICommonResponseCallback;
import im.zego.chathouse.protocol.ICreateRoomCallback;
import im.zego.chathouse.protocol.ILoginRoomCallback;
import im.zego.chathouse.protocol.RoomAPI;
import im.zego.chathouse.protocol.TokenManager;
import im.zego.chathouse.sdk.rtc.RTCSDKManager;
import im.zego.chathouse.sdk.rtc.callbacks.IRTCCommonCallback;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.chathouse.utils.ToastUtil;
import im.zego.enjoycommon.log.L;

/**
 * 登录服务，封装登录方法
 */
public class LoginService {

    public static final String TAG = "LoginService";

    private long[] init = new long[2];
    private int zimInit = -1;
    private int rtcInit = -1;



    /**
     * 创建并加入房间 （创建房间和加入房间只会执行其中一个，房主调用创建房间，其他人调用加入房间，需保证进房前版权音乐已经初始化完成）
     * 非可重入方法，在回调返回前，请勿多次调用
     * @param userName 用户名称
     * @param roomName 房间名称
     * @param cover 房间封面
     * @param application 上下文
     * @param callback 回调
     */
    public void createAndJoinRoom(String userName, String roomName, String cover, Application application, ILoginServiceCallback loginServicesCallback, ICommonResponseCallback callback) {

        TokenManager.getInstance().getToken(new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if(errorCode == 0){
                    RoomAPI.createRoom( roomName, cover, new ICreateRoomCallback() {
                        @Override
                        public void onCreateRoom(int retCode, RoomInfo roomInfo) {
                            if (retCode == 0) {
                                initSDK(UserManager.getInstance().getUserID(),userName,application, new IZIMCommonCallback() {
                                    @Override
                                    public void onZIMCallback(int errorCode) {

                                        //登录成功或者重复登录都认为是正常
                                        if(errorCode == 0 || errorCode == 6000111){

                                            LoginService.this.innerCreateAndJoinRoom(roomInfo.getRoomId(), roomInfo.getSubject(), loginServicesCallback, new IRTCCommonCallback() {
                                                @Override
                                                public void onRTCCallback(int errorCode) {

                                                    if (errorCode == 0) {
                                                        if (loginServicesCallback != null){
                                                            loginServicesCallback.loginRoomInitComplete(roomInfo);
                                                        }

                                                    }

                                                    if (callback != null) {
                                                        callback.onResponse(errorCode);
                                                    }
                                                    if (errorCode != 0) {
                                                        if (loginServicesCallback != null){
                                                            loginServicesCallback.loginRoomError(roomInfo.getRoomId());
                                                        }
                                                    }
                                                }
                                            });




                                        }else if(errorCode == ERROR_FAIL_NETWORK)
                                        {
                                            if(loginServicesCallback != null){
                                                loginServicesCallback.loginRoomError(roomInfo.getRoomId());
                                            }
                                            if(callback != null) {
                                                callback.onResponse(errorCode);
                                            }
//                    ToastUtil.showWarnToast(application.getResources().getString(R.string.login_tips_fail_network));
                                        } else {
                                            if(loginServicesCallback != null){
                                                loginServicesCallback.loginRoomError(roomInfo.getRoomId());
                                            }
                                            if(callback != null) {
                                                callback.onResponse(errorCode);
                                            }
                                            ToastUtil.showWarnToast(String.format(application.getString(R.string.chathouse_failed_to_join_room), String.valueOf(errorCode)));
                                        }
                                    }
                                });
                            } else {
                                if(loginServicesCallback != null){
                                    loginServicesCallback.loginRoomError(null);
                                }
                                if (callback != null) {
                                    callback.onResponse(retCode);
                                }

                                if (retCode == ERROR_FAIL_NETWORK) {
                                    ToastUtil.showWarnToast(R.string.room_list_join_fail_network);
                                }else if(retCode == ERROR_UID_HAS_CREATED_ROOM){
                                    ToastUtil.showWarnToast(application.getString(R.string.chathouse_users_can_only_create_one_room_at_time));
                                } else {
                                    ToastUtil.showWarnToast(String.format(application.getString(R.string.chathouse_failed_to_join_room), String.valueOf(retCode)));
                                }

                            }

                        }
                    });


                }else {
                    if (callback != null) {
                        callback.onResponse(errorCode);
                    }
                }


            }
        });



    }

    /**
     * 创建房间 （创建房间和加入房间只会执行其中一个，房主调用创建房间，其他人调用加入房间，需保证进房前版权音乐已经初始化完成）
     *
     * @param roomID 房间ID
     * @param roomName 房间名称
     * @param rtcCommonCallback 回调
     */
    private void innerCreateAndJoinRoom(String roomID, String roomName, ILoginServiceCallback loginServiceCallback, IRTCCommonCallback rtcCommonCallback) {
        clearInit();
        if(loginServiceCallback != null){
            loginServiceCallback.loginRoomBeforeInit(roomID);
        }
        ZIMSDKManager.getInstance().createRoom(roomID, roomName, new im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback() {
            @Override
            public void onZIMCallback(int errorCode) {
                // 错误码描述详见：https://doc-zh.zego.im/article/11606
                recordInitTime();
                zimInit = errorCode;
                initFinish(roomID, loginServiceCallback,rtcCommonCallback);

            }
        });

        RTCSDKManager.getInstance().loginRoom(roomID, new IRTCCommonCallback() {
            @Override
            public void onRTCCallback(int errorCode) {
                // 错误码描述详见：https://doc-zh.zego.im/article/4378
                recordInitTime();
                rtcInit = errorCode;
                initFinish(roomID, loginServiceCallback,rtcCommonCallback);
            }
        });

    }

    /**
     *
     * 加入房间
     * 非可重入方法，在回调返回前，请勿多次调用
     * @param roomID 房间ID
     * @param userName 用户名称
     * @param userIcon 头像
     * @param application 上下文
     * @param callback 回调
     */
    public void joinRoom(String roomID, String userName, String userIcon, Application application, ILoginServiceCallback loginServicesCallback, ICommonResponseCallback callback) {

        TokenManager.getInstance().getToken(new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if(errorCode == 0) {


                    RoomAPI.loginRoom(roomID, userName, userIcon, new ILoginRoomCallback() {
                        @Override
                        public void onLoginRoom(int errorCode, RoomInfo roomInfo) {
                            if (errorCode == 0) {
                                initSDK(UserManager.getInstance().getUserID(), userName, application, new IZIMCommonCallback() {
                                    @Override
                                    public void onZIMCallback(int errorCode) {
                                        //登录成功或者重复登录都认为是正常
                                        if (errorCode == 0 || errorCode == 6000111) {
                                            LoginService.this.joinRoom(roomID, loginServicesCallback, new IRTCCommonCallback() {
                                                @Override
                                                public void onRTCCallback(int errorCode) {
                                                    if (errorCode == 0) {
                                                        if (loginServicesCallback != null) {
                                                            loginServicesCallback.loginRoomInitComplete(roomInfo);
                                                        }

                                                    }

                                                    if (callback != null) {
                                                        callback.onResponse(errorCode);
                                                    }

                                                    if (errorCode != 0) {
                                                        if (loginServicesCallback != null) {
                                                            loginServicesCallback.loginRoomError(roomInfo.getRoomId());
                                                        }
                                                    }
                                                }
                                            });
                                        } else {
                                            if (loginServicesCallback != null) {
                                                loginServicesCallback.loginRoomError(roomInfo.getRoomId());
                                            }
                                            if (callback != null) {
                                                callback.onResponse(errorCode);
                                            }
//                                            ToastUtil.showWarnToast(String.format(application.getString(R.string.chathouse_failed_to_join_error_code), errorCode));
                                        }
                                    }
                                });
                            } else if (errorCode == ERROR_FAIL_NETWORK) {
                                if (loginServicesCallback != null) {
                                    loginServicesCallback.loginRoomError(null);
                                }
                                if (callback != null) {
                                    callback.onResponse(errorCode);
                                }
//                                ToastUtil.showWarnToast(application.getResources().getString(R.string.login_tips_fail_network));
                            } else if (errorCode == ERROR_ROOM_FULL) {
                                if (loginServicesCallback != null) {
                                    loginServicesCallback.loginRoomError(null);
                                }
                                if (callback != null) {
                                    callback.onResponse(errorCode);
                                }
                                ToastUtil.showWarnToast(application.getString(R.string.chathouse_room_is_full_please_change_or_create_a_new_room));
                            } else if (errorCode == ERROR_ROOM_NOT_EXIST) {
                                if (loginServicesCallback != null) {
                                    loginServicesCallback.loginRoomError(null);
                                }
                                if (callback != null) {
                                    callback.onResponse(errorCode);
                                }
                                ToastUtil.showWarnToast(application.getString(R.string.chathouse_room_no_exist_please_refresh_the_list));
                            } else {
                                if (loginServicesCallback != null) {
                                    loginServicesCallback.loginRoomError(null);
                                }
                                if (callback != null) {
                                    callback.onResponse(errorCode);
                                }
//                                ToastUtil.showWarnToast(String.format(application.getString(R.string.chathouse_failed_to_join_error_code), errorCode));
                            }
                        }
                    });


                }
                else {
                    if (callback != null) {
                        callback.onResponse(errorCode);
                    }
                }


            }
        });


    }


    /**
     * 加入房间 （创建房间和加入房间只会执行其中一个，房主调用创建房间，其他人调用加入房间，需保证进房前版权音乐已经初始化完成）
     *
     * @param roomID 房间ID
     * @param rtcCommonCallback 回调
     */
    private void joinRoom(String roomID, ILoginServiceCallback loginServicesCallback, IRTCCommonCallback rtcCommonCallback) {
        clearInit();
        if(loginServicesCallback != null){
            loginServicesCallback.loginRoomBeforeInit(roomID);
        }
        ZIMSDKManager.getInstance().joinRoom(roomID, new im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback() {
            @Override
            public void onZIMCallback(int errorCode) {
                // 错误码描述详见：https://doc-zh.zego.im/article/11606
                // 如果进房失败可能是房间已经被销毁，尝试创建新房间
                if(errorCode == 6000302 || errorCode == 6000304 ) {
                    ZIMSDKManager.getInstance().createRoom(roomID, "null", new im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback() {
                        @Override
                        public void onZIMCallback(int errorCode) {
                            recordInitTime();
                            zimInit = errorCode;
                            initFinish(roomID,loginServicesCallback, rtcCommonCallback);
                        }
                    });
                }else {

                    //6000320 用户已经在房间内 ，也认为成功
                    if(errorCode == 6000320){
                        errorCode = 0;
                    }

                    recordInitTime();
                    zimInit = errorCode;
                    initFinish(roomID,loginServicesCallback, rtcCommonCallback);
                }
            }
        });

        RTCSDKManager.getInstance().loginRoom(roomID, new IRTCCommonCallback() {
            @Override
            public void onRTCCallback(int errorCode) {
                // 错误码描述详见：https://doc-zh.zego.im/article/4378
                recordInitTime();
                rtcInit = errorCode;
                initFinish(roomID, loginServicesCallback,rtcCommonCallback);
            }
        });
    }

    /**
     * 初始化SDK(RTC+ZIM), 登录ZIM(login)
     * @param userID 用户ID
     * @param userName 用户昵称
     * @param application 上下文
     * @param sdkCommonCallback 回调
     */
    private void initSDK(String userID, String userName, Application application, IZIMCommonCallback sdkCommonCallback){
        ZIMSDKManager.getInstance().init(application);
        RTCSDKManager.getInstance().init(application);
//        TokenManager.getInstance().getToken(new ICommonResponseCallback() {
//            @Override
//            public void onResponse(int errorCode) {
//                if(errorCode == 0){
        ZIMSDKManager.getInstance().login(userID, userName, TokenManager.getInstance().getToken(),new im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback() {
            @Override
            public void onZIMCallback(int errorCode) {
                //登录成功或者重复登录都认为是正常
//                            if(errorCode == 0 || errorCode == 6000111){
//                                T.show("登录成功");
//                            }else{
//                                T.show("登录失败");
//                            }

                if(sdkCommonCallback != null){
                    sdkCommonCallback.onZIMCallback(errorCode);
                }

            }
        });
//                }else {
////                    T.show(application.getString(R.string.chathouse_get_token_failed));
//                    if(sdkCommonCallback != null){
//                        sdkCommonCallback.onZIMCallback(errorCode);
//                    }
//                }
//
//
//            }
//        });

    }

    /**
     * 当初始化数组中首位也被记录了，则表示所有SDK都已经初始化完成
     *
     * @param roomID 房间ID
     * @param rtcCommonCallback 回调
     */
    public void initFinish(String roomID, ILoginServiceCallback loginServicesCallback, IRTCCommonCallback rtcCommonCallback) {
        //当数组第0位都不为0，则表示几个都初始化完成
        if (init != null && init[0] != 0) {
            if (zimInit == 0 && rtcInit == 0 ) {
//                T.show("初始化成功");

                if (rtcCommonCallback != null) {
                    rtcCommonCallback.onRTCCallback(0);
                }

//                loginRoomInit();

                return;
            }
//            else {

//                if(loginServicesCallback != null){
//                    loginServicesCallback.loginRoomError(roomID);
//                }
//                loginRoomError();
//            }

            if (zimInit != 0) {
//                T.show(String.format(MiscUtils.getApplication().getString(R.string.chathouse_im_entry_failed), zimInit));
                L.e(TAG, "zim init error :" + zimInit);
                if (rtcCommonCallback != null) {
                    rtcCommonCallback.onRTCCallback(zimInit);
                }

            }

            if (rtcInit != 0) {
//                T.show(String.format(MiscUtils.getApplication().getString(R.string.chathouse_rtc_entry_failed), rtcInit));
                L.e(TAG, "ryc init error :" + rtcInit);
                if (rtcCommonCallback != null) {
                    rtcCommonCallback.onRTCCallback(rtcInit);
                }
            }

            RTCSDKManager.getInstance().logoutRoom();
            ZIMSDKManager.getInstance().logoutRoom(roomID, null);
        }
    }

    /**
     * 清除初始化数据
     */
    public void clearInit() {
        init = new long[2];
        zimInit = -1;
        rtcInit = -1;
    }

    /**
     * 记录初始化时间,每次初始化后都用数组中一个数记录下当前时间
     */
    private void recordInitTime() {
        System.arraycopy(init, 1, init, 0, init.length - 1);
        init[init.length - 1] = SystemClock.uptimeMillis();
    }



}
