package im.zego.chathouse.room;

import android.app.Application;
import android.text.TextUtils;

import java.util.HashMap;
import java.util.Map;

import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.room.callback.ILoginServiceCallback;
import im.zego.chathouse.room.callback.IRoomInitStateCallback;
import im.zego.chathouse.room.callback.IServicesNotNullCallback;
import im.zego.chathouse.model.RoomInfo;
import im.zego.chathouse.protocol.ICommonResponseCallback;
import im.zego.chathouse.protocol.RoomAPI;
import im.zego.chathouse.room.service.DialogQueueService;
import im.zego.chathouse.room.service.InviteService;
import im.zego.chathouse.room.service.RoomAttributesService;
import im.zego.chathouse.room.service.RoomConnectService;
import im.zego.chathouse.room.service.RoomDialogService;
import im.zego.chathouse.room.service.RoomInfoService;
import im.zego.chathouse.room.service.RoomStreamService;
import im.zego.chathouse.room.service.RunningRoomService;
import im.zego.chathouse.room.service.TimerTaskService;
import im.zego.chathouse.room.service.UserService;
import im.zego.chathouse.sdk.rtc.RTCSDKManager;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.enjoycommon.log.L;


public class RoomManager {

    public static final String TAG = "ChatRoomManager";

    private static volatile RoomManager singleton = null;


    /**
     * 房间ID对应的房间服务
     */
    private final Map<String, RunningRoomService> mRoomServicesMap = new HashMap<>();

    /**
     * 登录服务
     */
    private final LoginService mLoginService = new LoginService();

    /**
     * 当前房间ID
     */
    private String mCurrentRoomID = null;

    /**
     * 房间初始化状态
     */
    private RoomInitState mRoomInitState = RoomInitState.UNINITIALIZED;

    private IRoomInitStateCallback mRoomInitStateCallback;


    private RoomManager() {
    }

    public static RoomManager getInstance() {
        if (singleton == null) {
            synchronized (RoomManager.class) {
                if (singleton == null) {
                    singleton = new RoomManager();
                }
            }
        }
        return singleton;
    }

    /**
     * 设置当前房间ID
     * @param currentRoomID 房间ID
     */
    public void setCurrentRoomID(String currentRoomID) {
        this.mCurrentRoomID = currentRoomID;
    }

    /**
     * 返回当前房间ID
     */
    public String getCurrentRoomID() {
        return this.mCurrentRoomID == null? "":this.mCurrentRoomID;
    }

    /**
     * 创建并加入房间
     * @param userName 用户昵称
     * @param roomName 房间名称
     * @param cover 房间封面
     * @param application 上下文
     * @param callback 回调
     */
    public void createAndJoinRoom(String userName,String roomName, String cover, Application application,  ICommonResponseCallback callback){
        setRoomInitState(RoomInitState.INITIALIZING);
        mLoginService.createAndJoinRoom(userName,roomName,cover,application,new ILoginServiceCallback(){
            @Override
            public void loginRoomBeforeInit(String roomID) {
                setCurrentRoomID( roomID);
                RoomManager.this.loginRoomBeforeInit(roomID);
            }

            @Override
            public void loginRoomInitComplete(RoomInfo roomInfo) {
                applyRoomInfoServices(
                        services -> services.setRoomInfo(roomInfo)
                );
                RoomManager.this.loginRoomInit();
                setRoomInitState(RoomInitState.INIT_COMPLETED);
            }

            @Override
            public void loginRoomError(String roomID) {
                if(roomID != null){
                    RoomAPI.closeRoom(roomID, null);
                }
                RoomManager.this.loginRoomError();
                unInit();
                setRoomInitState(RoomInitState.INIT_ERROR);
            }
        },callback);
    }

    /**
     * 加入房间
     * @param roomID 房间ID
     * @param userName 用户名称
     * @param userIcon 用户头像
     * @param application 上下文
     * @param callback 回调
     */
    public void joinRoom(String roomID, String userName, String userIcon, Application application, ICommonResponseCallback callback){
        setRoomInitState(RoomInitState.INITIALIZING);
        mLoginService.joinRoom(roomID,userName,userIcon,application,new ILoginServiceCallback(){
            @Override
            public void loginRoomBeforeInit(String roomID) {
                setCurrentRoomID( roomID);
                RoomManager.this.loginRoomBeforeInit(roomID);
            }

            @Override
            public void loginRoomInitComplete(RoomInfo roomInfo) {
                applyRoomInfoServices(
                        services -> services.setRoomInfo(roomInfo)
                );
                RoomManager.this.loginRoomInit();
                setRoomInitState(RoomInitState.INIT_COMPLETED);
            }

            @Override
            public void loginRoomError(String roomID) {
                if(roomID != null){
                    RoomAPI.quitRoom(roomID, null);
                }
                RoomManager.this.loginRoomError();
                unInit();
                setRoomInitState(RoomInitState.INIT_ERROR);
            }
        },callback);
    }

    /**
     * 进入房间前初始化
     * 在进入房间前需要设置回调对象
     * 保证登录房间后返回的信息不丢失
     */
    private void loginRoomBeforeInit(String roomID){

        if(!TextUtils.isEmpty(roomID)) {
            RunningRoomService runningRoomService = mRoomServicesMap.get(roomID);
            if(runningRoomService != null) {
                runningRoomService.destroyServices();
            }
            runningRoomService = new RunningRoomService(roomID);
            runningRoomService.createServices();
            mRoomServicesMap.put(roomID, runningRoomService);
        }
    }


    /**
     * 进房成功后初始化
     */
    private void loginRoomInit() {

        applyRunningRoomService(
                RunningRoomService::initRoomComplete
        );
    }

    /**
     * 进入房间失败后操作
     */
    private void loginRoomError(){

        logoutRoom();
    }

    /**
     * 退出房间
     * 对SDK执行退房操作
     * 反初始化工具类
     * 清除说有管理类的退房数据
     */
    public void logoutRoom() {

        applyRoomInfoServices(
                services -> logoutRoom(services.getRoomID())
        );

        ZIMSDKManager.getInstance().getDispatchService().clearZIMEventHandlerCallbacksByRoomID(mCurrentRoomID);
        RTCSDKManager.getInstance().getDispatchService().clearRTCEventHandlerCallbacksByRoomID(mCurrentRoomID);
        RunningRoomService runningRoomService = mRoomServicesMap.remove(mCurrentRoomID);
        if(runningRoomService != null){
            runningRoomService.destroyServices();
        }
        setCurrentRoomID(null);
    }

    /**
     * 退出房间
     * @param roomID 房间ID
     */
    private void logoutRoom(String roomID) {
        RTCSDKManager.getInstance().logoutRoom();
        ZIMSDKManager.getInstance().logoutRoom(roomID, new IZIMCommonCallback() {
            @Override
            public void onZIMCallback(int errorCode) {
                // 错误码描述详见：https://doc-zh.zego.im/article/11606

            }
        });
    }

    /**
     * 离开后台房间
     * @param callback 回调
     */
    public void quitRoom(ICommonResponseCallback callback) {

        applyRoomInfoServices(
                services -> {
                    RoomAPI.quitRoom(services.getRoomID(), new ICommonResponseCallback() {
                        @Override
                        public void onResponse(int errorCode) {
                            if (callback != null) {
                                callback.onResponse(errorCode);
                            }
                        }
                    });
                }
        );

    }

    /**
     * 关闭后台房间
     * @param callback 回调
     */
    public void closeRoom(ICommonResponseCallback callback) {

        applyRoomInfoServices(
                services -> {
                    RoomAPI.closeRoom(services.getRoomID(), new ICommonResponseCallback() {
                        @Override
                        public void onResponse(int errorCode) {
                            if (callback != null) {
                                callback.onResponse(errorCode);
                            }
                        }
                    });
                }
        );

    }


    /**
     * 申请当前房间服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyRunningRoomService(IServicesNotNullCallback<RunningRoomService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null && callback != null){
                callback.block(services);
            }
        }

    }

    /**
     * 申请当前房间的房间信息服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyRoomInfoServices(IServicesNotNullCallback<RoomInfoService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                services.applyRoomInfoServices(callback);
            }
        }

    }

    /**
     * 申请当前房间的房间连接服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyRoomConnectService(IServicesNotNullCallback<RoomConnectService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                services.applyRoomConnectService(callback);
            }
        }

    }

    /**
     * 申请当前房间的房间属性服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyRoomAttributesService(IServicesNotNullCallback<RoomAttributesService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                services.applyRoomAttributesService(callback);
            }
        }

    }

    /**
     * 申请当前房间的用户服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyUserService(IServicesNotNullCallback<UserService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                services.applyUserService(callback);
            }
        }

    }

    /**
     * 申请当前房间的定时任务服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyTimerTaskService(IServicesNotNullCallback<TimerTaskService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                services.applyTimerTaskService(callback);
            }
        }

    }

    /**
     * 获取当前房间的用户服务，服务可能为空
     * @return 用户服务
     */
    public UserService getUserService() {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                return services.getUserService();
            }
        }

        return null;

    }

    /**
     * 申请当前房间的邀请服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyInviteService(IServicesNotNullCallback<InviteService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                services.applyInviteService(callback);
            }
        }

    }

    /**
     * 申请当前房间的弹窗队列服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyDialogQueueService(IServicesNotNullCallback<DialogQueueService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                services.applyDialogQueueService(callback);
            }
        }

    }

    /**
     * 申请当前房间的弹窗服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyRoomDialogService(IServicesNotNullCallback<RoomDialogService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                services.applyRoomDialogService(callback);
            }
        }

    }

    /**
     * 申请当前房间的房间流服务，服务不为空时进行回调
     * @param callback 回调
     */
    public void applyRoomStreamService(IServicesNotNullCallback<RoomStreamService> callback) {

        if(mCurrentRoomID != null){
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if(services != null){
                services.applyRoomStreamService(callback);
            }
        }

    }

    /**
     * 判断自己是否是房主
     * @return true 是房主
     */
    public boolean isHost() {
        if (mCurrentRoomID != null) {
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if (services != null) {
                RoomInfoService roomInfoService = services.getRoomInfoService();
                if (roomInfoService != null && roomInfoService.getRoomInfo() != null) {
                    return UserManager.getInstance().getUserID().equals(roomInfoService.getRoomInfo().getCreatorId());
                }
            }
        }
        return false;
    }

    /**
     * 判断用户是否是房主
     * @param userID 用户ID
     * @return true 是房主
     */
    public boolean isHost(String userID) {
        if (mCurrentRoomID != null && !TextUtils.isEmpty(userID)) {
            RunningRoomService services = mRoomServicesMap.get(mCurrentRoomID);
            if (services != null) {
                RoomInfoService roomInfoService = services.getRoomInfoService();
                if (roomInfoService != null && roomInfoService.getRoomInfo() != null) {
                    return userID.equals(roomInfoService.getRoomInfo().getCreatorId());
                }
            }
        }
        return false;
    }


    /**
     * 反初始化SDK
     * 注意调用时机，如果在初始化未完成的情况下调用，版权音乐反初始化可能导致崩溃
     */
    public void unInit(){
        L.i(TAG,"unInit");
        ZIMSDKManager.getInstance().logout();
        ZIMSDKManager.getInstance().unInit();
        RTCSDKManager.getInstance().unInit();
    }

    public void setRoomInitState(RoomInitState roomInitState){
        mRoomInitState = roomInitState;
        if(mRoomInitStateCallback != null){
            mRoomInitStateCallback.onRoomInitStateCallback(mRoomInitState);
        }
    }

    public void setRoomInitStateCallback(IRoomInitStateCallback roomInitStateCallback) {
        this.mRoomInitStateCallback = roomInitStateCallback;
        if(mRoomInitStateCallback != null){
            mRoomInitStateCallback.onRoomInitStateCallback(mRoomInitState);
        }
    }

    public RoomInitState getRoomInitState(){
        return mRoomInitState;
    }


    public enum RoomInitState{
        /**
         * 未初始化
         */
        UNINITIALIZED,
        /**
         * 初始化中
         */
        INITIALIZING,
        /**
         * 初始化完成
         */
        INIT_COMPLETED,
        /**
         * 初始化错误
         */
        INIT_ERROR,
    }

}
