package im.zego.chathouse.room.service;

import static im.zego.chathouse.model.RoomAttributes.*;
import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.DATA_PRIORITY;

import android.os.SystemClock;
import android.text.TextUtils;
import android.util.Log;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import im.zego.chathouse.R;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.RoomAttributes;
import im.zego.chathouse.model.Seat;
import im.zego.chathouse.model.User;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.room.help.TimerTaskHelp;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;

import im.zego.chathouse.sdk.zim.callback.IZIMQueryRoomMemberListCallback;
import im.zego.enjoycommon.log.L;

import im.zego.enjoycommon.util.T;
import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMRoomAttributesQueriedCallback;
import im.zego.zim.entity.ZIMError;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;
import im.zego.zim.entity.ZIMUserFullInfo;
import im.zego.zim.entity.ZIMUserInfo;
import im.zego.zim.enums.ZIMRoomAttributesUpdateAction;
import im.zego.zim.enums.ZIMRoomEvent;
import im.zego.zim.enums.ZIMRoomState;

/**
 * 房间属性服务（麦位管理服务）
 * 房间属性包含5个麦位的信息，其中0固定为房主，不会改变
 */
public class RoomAttributesService {

    /**
     * 房间ID
     */
    private final String mRoomID;
    /**
     * LOG 标志
     */
    private static final String TAG = "RoomAttributesService";

    /**
     * 是否能更新房间属性
     */
    private boolean mCanUpdateRoomAttributes = true;
    /**
     * 记录房间属性
     */
    RoomAttributes mRoomAttributes = new RoomAttributes();

    private final ZIMEventHandlerWrap zimEventHandlerWrap;

    public RoomAttributesService(String roomID){
        mRoomID = roomID;
        mCanUpdateRoomAttributes = true;
        zimEventHandlerWrap = new ZIMEventHandlerWrap(mRoomID,DATA_PRIORITY) {
            @Override
            public void onRoomAttributesUpdated(ZIM zim, ZIMRoomAttributesUpdateInfo info, String roomID) {
                super.onRoomAttributesUpdated(zim, info, roomID);
                //房间属性变更后刷新麦上列表和麦上成员信息
                HashMap<String,String> map = info.roomAttributes;
                if(map != null) {
                    for (Map.Entry<String, String> entry : map.entrySet()) {
                        Log.i(TAG, "onRoomAttributesUpdated  key : " + entry.getKey() + ": value :" + entry.getValue());
                    }
                }

                parseAttributes(map);

            }

            @Override
            public void onRoomMemberLeft(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                super.onRoomMemberLeft(zim, memberList, roomID);
                if(memberList != null) {
                    //监听用户离开后，发现是麦上的人员则进行下麦
                    for (ZIMUserInfo zimUserInfo : memberList) {
                        for (int i = 0; i < SEAT_LIST.length; i++) {
                            User user = mRoomAttributes.getSeat(i).getUser();
                            if (user != null && user.getId() != null) {

                                if(user.getId().equals(zimUserInfo.userID)){
                                    downSeat(i);
                                }

                            }
                        }
                    }
                }

            }

            @Override
            public void onRoomStateChanged(ZIM zim, ZIMRoomState state, ZIMRoomEvent event, JSONObject extendedData, String roomID){

                if(RoomManager.getInstance().getRoomInitState() == RoomManager.RoomInitState.INIT_COMPLETED) {

                    if (state == ZIMRoomState.CONNECTING) {

                        setCanUpdateRoomAttributes(false);

                    } else if (state == ZIMRoomState.CONNECTED) {

                        ZIMSDKManager.getInstance().queryRoomAllAttributes(mRoomID, (roomID1, roomAttributes, errorInfo) -> {
                            //模拟一次房间属性更新
                            L.i(TAG,"onRoomAttributesQueried : value = " + errorInfo.code.value());
                            if(errorInfo.code.value() == 0) {
                                ZIMRoomAttributesUpdateInfo zimRoomAttributesUpdateInfo = new ZIMRoomAttributesUpdateInfo();
                                zimRoomAttributesUpdateInfo.roomAttributes = roomAttributes;
                                zimRoomAttributesUpdateInfo.action = ZIMRoomAttributesUpdateAction.SET;

                                ZIMSDKManager.getInstance().getDispatchService().onRoomAttributesUpdated(ZIMSDKManager.getInstance().zim, zimRoomAttributesUpdateInfo, mRoomID);
                            }
                            setCanUpdateRoomAttributes(true);
                        });

                    }
                }

            }


        };

    }

    public void init(){
        ZIMSDKManager.getInstance().getDispatchService().registerZIMEventHandlerCallbacks(zimEventHandlerWrap);
    }


    public void unInit(){
        ZIMSDKManager.getInstance().getDispatchService().unregisterZIMEventHandlerCallbacks(zimEventHandlerWrap);
    }

    /**
     * 房主初始化
     */
    public void hostInit(){


        Seat seat = mRoomAttributes.getSeat(0);
        User user = seat.getUser();
        if(user == null){
            user = new User();
            seat.setUser(user);
        }
        user.setIdx(0);
        user.setMicOn(true);
        user.setUserIcon(UserManager.getInstance().getUserIcon());
        user.setId(UserManager.getInstance().getUserID());
        user.setName(UserManager.getInstance().getUserName());
        user.setRole(User.UserRole.HOST.role);
        user.setCanSpeak(true);
        seat.setIdx(0);
        seat.setStatus(Seat.SeatStatus.OCCUPIED.status);

        updateRoomAttributes();
    }

    /**
     * 打开麦位
     * @param position 麦位
     */
    public void openSeat(int position) {
        if(!RoomManager.getInstance().isHost()){
            L.e(TAG,"openMic permission error");
            return;
        }

        L.i(TAG,"openSeat:" + position);

        Seat seat = mRoomAttributes.getSeat(position);
        seat.setStatus(Seat.SeatStatus.VACANT.status);
        updateRoomAttributes();
    }

    /**
     * 关闭麦位
     * @param position 麦位
     */
    public void closeSeat(int position){
        if(!RoomManager.getInstance().isHost()){
            L.e(TAG,"closeMic permission error");
            return;
        }

        L.i(TAG,"closeSeat:" + position);

        Seat seat = mRoomAttributes.getSeat(position);
        seat.setStatus(Seat.SeatStatus.CLOSED.status);
        updateRoomAttributes();
    }

    /**
     * 邀请麦位
     * @param position 麦位
     * @param userID 用户ID
     */
    public void invitingSeat(int position, String userID){
        if(!RoomManager.getInstance().isHost()){
            L.e(TAG,"requestMic permission error");
            return;
        }

        L.i(TAG,"invitingSeat:  position = " + position + ": userID = " + userID);

        Seat seat = mRoomAttributes.getSeat(position);

        if(seat.getStatus() == Seat.SeatStatus.CLOSED.status){
            return;
        }

        RoomManager.getInstance().applyUserService(
                services -> {
                    User user = services.getUser(userID);
                    if(user != null){
                        //如果不存在用户头像，则去查找后再设置
                        if(!TextUtils.isEmpty(user.getUserIcon())){
                            innerInvite(position,user);
                            //设置用户信息对应的状态
                            services.setUserStatus(user.getId(),User.Status.INVITING.status);
                        }else {
                            ArrayList<String> userList = new ArrayList<>();
                            userList.add(userID);
                            ZIMSDKManager.getInstance().queryUsersInfo(userList, new IZIMQueryRoomMemberListCallback() {
                                @Override
                                public void onRoomMemberQueried(ArrayList<ZIMUserFullInfo> userList, String nextFlag, int errorCode) {
                                    if(errorCode == 0 && userList != null && userList.size() >= 1){
                                        ZIMUserFullInfo info = userList.get(0);
                                        user.setUserIcon(info.userAvatarUrl);
                                    }
                                    innerInvite(position,user);
                                    //设置用户信息对应的状态
                                    services.setUserStatus(user.getId(),User.Status.INVITING.status);
                                }
                            });
                        }
                    }
                }
        );
    }


    /**
     * 开关麦克风
     * @param userID 用户ID
     */
    public void setMicOnOff( String userID,boolean open){
        if(!RoomManager.getInstance().isHost()){
            L.e(TAG,"setMicOnOff permission error");
            return;
        }

        if(TextUtils.isEmpty(userID)){
            return;
        }

        L.i(TAG,"setMicOnOff:  userID = " + userID + ": open = " + open);

        for(int i = 0; i < SEAT_LIST.length;i++){
            Seat seat = mRoomAttributes.getSeat(i);
            if(seat != null && seat.getUser() != null && userID.equals(seat.getUser().getId())){
                seat.getUser().setMicOn(open);
            }
        }

        updateRoomAttributes();

    }


    /**
     * 设置能否打开麦克风
     * @param position 麦位
     * @param canSpeak 能否打开麦克风（true 能 false 不能）
     * @param userID 用户ID
     */
    public void canSpeak(int position,boolean canSpeak,String userID){
        if(!RoomManager.getInstance().isHost()){
            L.e(TAG,"canSpeak permission error");
            return;
        }

        L.i(TAG,"canSpeak:  position = " + position + ": canSpeak = " + canSpeak + ": userID = " + userID);

        Seat seat = mRoomAttributes.getSeat(position);

        if(seat.getStatus() != Seat.SeatStatus.OCCUPIED.status){
            return;
        }

        if(seat.getUser() == null){
            return;
        }

        RoomManager.getInstance().applyUserService(
                services -> {
                    //获取麦位对应的用户
                    User user = seat.getUser();
                    if(user != null){
                        if(!TextUtils.isEmpty(user.getUserIcon())){
                            user.setCanSpeak(canSpeak);
                            updateRoomAttributes();
                        }else {
                            ArrayList<String> userList = new ArrayList<>();
                            userList.add(userID);
                            ZIMSDKManager.getInstance().queryUsersInfo(userList, new IZIMQueryRoomMemberListCallback() {
                                @Override
                                public void onRoomMemberQueried(ArrayList<ZIMUserFullInfo> userList, String nextFlag, int errorCode) {
                                    if(errorCode == 0 && userList != null && userList.size() >= 1){
                                        ZIMUserFullInfo info = userList.get(0);
                                        user.setUserIcon(info.userAvatarUrl);
                                    }
                                    user.setCanSpeak(canSpeak);
                                    updateRoomAttributes();

                                }
                            });
                        }
                    }
                }
        );
    }

    /**
     * 获取当前麦位的麦克风打开权限
     * @param position 麦位
     * @return 0表示没有，1表示有，-1表示不存在
     */
    public int getCanSpeak(int position){
        Seat seat = mRoomAttributes.getSeat(position);
        if(seat != null && seat.getStatus() == Seat.SeatStatus.OCCUPIED.status && seat.getUser() != null){
            User user = seat.getUser();
            return user.isCanSpeak() ? 1: 0 ;
        }

        return -1;
    }


    /**
     * 下麦
     * @param position 麦位
     */
    public void downSeat(int position){
        if(!RoomManager.getInstance().isHost()){
            L.e(TAG,"downMic permission error");
            return;
        }

        L.i(TAG,"downSeat:  position = " + position );

        Seat seat = mRoomAttributes.getSeat(position);

        if(seat.getStatus() == Seat.SeatStatus.CLOSED.status){
            return;
        }

        seat.setUser(null);
        seat.setStatus(Seat.SeatStatus.VACANT.status);

        //下麦移除操作弹窗
        RoomManager.getInstance().applyRoomDialogService(services -> services.dismissOperationDialog(position));

        updateRoomAttributes();
    }

    /**
     * 非房主下麦
     * @param position 麦位
     */
    public void stageGuestDownSeat(int position,IZIMCommonCallback callback ){

        L.i(TAG,"stageGuestDownSeat:  position = " + position );

        Seat seat = mRoomAttributes.getSeat(position);

        if(seat.getStatus() == Seat.SeatStatus.CLOSED.status){
            return;
        }

        updateRoomAttributesBySeat(seat.getIdx(), new IZIMCommonCallback() {
            @Override
            public void onZIMCallback(int errorCode) {
                if(errorCode == 0){
                }
                if(callback != null){
                    callback.onZIMCallback(errorCode);
                }
            }
        });
    }

    /**
     * 上麦
     * @param position 麦位
     * @param userID 用户ID
     */
    public void upSeat(int position, String userID){

        if(!RoomManager.getInstance().isHost()){
            L.e(TAG,"upMic permission error");
            return;
        }

        L.i(TAG,"upSeat:  position = " + position + ": userID = " + userID);

        Seat seat = mRoomAttributes.getSeat(position);

        if(seat.getStatus() == Seat.SeatStatus.CLOSED.status){
            return;
        }

        RoomManager.getInstance().applyUserService(
                services -> {
                    User user = services.getUser(userID);
                    if(user != null){
                        if(!TextUtils.isEmpty(user.getUserIcon())){
                            innerUpSeat(position,user);
                            services.setUserStatus(user.getId(),User.Status.OCCUPIED.status);
                        }else {
                            ArrayList<String> userList = new ArrayList<>();
                            userList.add(userID);
                            ZIMSDKManager.getInstance().queryUsersInfo(userList, new IZIMQueryRoomMemberListCallback() {
                                @Override
                                public void onRoomMemberQueried(ArrayList<ZIMUserFullInfo> userList, String nextFlag, int errorCode) {
                                    if(errorCode == 0 && userList != null && userList.size() >= 1){
                                        ZIMUserFullInfo info = userList.get(0);
                                        user.setUserIcon(info.userAvatarUrl);

                                    }
                                    innerUpSeat(position,user);
                                    services.setUserStatus(user.getId(),User.Status.OCCUPIED.status);
                                }
                            });
                        }
                    }
                }
        );
    }

    /**
     * 设置麦位为邀请状态并更新
     * @param position 麦位
     * @param user 用户信息
     */
    public void innerInvite(int position,User user){
        if(!RoomManager.getInstance().isHost()){
            L.e(TAG,"requestMic permission error");
            return;
        }

        if(user == null){
            return;
        }

        L.i(TAG,"innerInvite:  position = " + position + ": userID = " + user.getId());
        Seat seat = mRoomAttributes.getSeat(position);

        seat.setStatus(Seat.SeatStatus.INVITING.status);
        User seatUser = seat.getUser();
        if(seatUser == null){
            seatUser = new User();
            seat.setUser(seatUser);
        }
        seatUser.setId(user.getId());
        seatUser.setName(user.getName());
        seatUser.setUserIcon(user.getUserIcon());
        seatUser.setRole(User.UserRole.AUDIENCE.role);
        seatUser.setMicOn(true);
        seatUser.setIdx(position);
        //上麦则能默认允许，即使之前被禁止了
        seatUser.setCanSpeak(true);

        user.setInviteTime(SystemClock.elapsedRealtime());
        user.setRole(User.UserRole.AUDIENCE.role);

        updateRoomAttributes();

    }


    /**
     * 上麦并更新房间属性
     * @param position 麦位
     * @param user 用户
     */
    public void innerUpSeat(int position, User user){
        if(!RoomManager.getInstance().isHost()){
            L.e(TAG,"requestMic permission error");
            return;
        }

        if(user == null){
            return;
        }

        L.i(TAG,"innerUpSeat:  position = " + position + ": userID = " + user.getId());
        Seat seat = mRoomAttributes.getSeat(position);

        seat.setStatus(Seat.SeatStatus.OCCUPIED.status);
        User seatUser = seat.getUser();
        if(seatUser == null){
            seatUser = new User();
            seat.setUser(seatUser);
        }
        seatUser.setId(user.getId());
        seatUser.setName(user.getName());
        seatUser.setUserIcon(user.getUserIcon());
        seatUser.setRole(User.UserRole.GUEST.role);
        seatUser.setMicOn(true);
        seatUser.setIdx(position);
        //上麦则能默认允许，即使之前被禁止了
        seatUser.setCanSpeak(true);

        user.setRole(User.UserRole.GUEST.role);
        user.setInviteTime(SystemClock.elapsedRealtime());
        updateRoomAttributes();

    }

    /**
     * 更新房间属性信息并提交
     */
    public void updateRoomAttributes(){
        mRoomAttributes.getInfo().setAuthor(UserManager.getInstance().getUserID());

        pushRoomAttributes(new IZIMCommonCallback() {
            @Override
            public void onZIMCallback(int errorCode) {
                //设置房间属性失败则开启重试
                if(errorCode != 0){
                    TimerTaskHelp.pushRoomAttributes();
                }
            }
        });

    }

    /**
     * 非房主更新房间属性信息并提交
     */
    public void updateRoomAttributesBySeat(int position,IZIMCommonCallback callback ){

        pushRoomAttributesBySeat(position,new IZIMCommonCallback() {
            @Override
            public void onZIMCallback(int errorCode) {

                if(callback != null){
                    callback.onZIMCallback(errorCode);
                }

            }
        });

    }

    /**
     * 设置房间属性并重试
     */
    public void pushRoomAttributes(IZIMCommonCallback callback){
        RoomManager.getInstance().applyRoomInfoServices(
                services -> {
                    ZIMSDKManager.getInstance().setRoomAttributes(mRoomAttributes.getSerializeMap(),
                            services.getRoomID(), new IZIMCommonCallback() {
                                @Override
                                public void onZIMCallback(int errorCode) {
                                    if(callback != null){
                                        callback.onZIMCallback(errorCode);
                                    }
                                }
                            });
                }
        );
    }

    /**
     * 设置房间麦位并重试
     */
    public void pushRoomAttributesBySeat(int position , IZIMCommonCallback callback){
        RoomManager.getInstance().applyRoomInfoServices(
                services -> {

                    if (position > 0 && position <= SEAT_LIST.length) {
                        String seatTag = RoomAttributes.SEAT_LIST[position];

                        Seat seat = mRoomAttributes.getSeat(position);

                        Seat tempSeat = new Seat();
                        tempSeat.setIdx(seat.getIdx());
                        tempSeat.setUser(null);
                        tempSeat.setStatus(Seat.SeatStatus.VACANT.status);

                        Info info = mRoomAttributes.getInfo();
                        Info tempInfo = new Info();
                        tempInfo.setAuthor(UserManager.getInstance().getUserID());
                        ZIMSDKManager.getInstance().setRoomAttributes(mRoomAttributes.getSerializeMap(seatTag,tempSeat,tempInfo),
                                services.getRoomID(), new IZIMCommonCallback() {
                                    @Override
                                    public void onZIMCallback(int errorCode) {
                                        if (callback != null) {
                                            callback.onZIMCallback(errorCode);
                                        }
                                    }
                                });
                    }
                }
        );
    }

    /**
     * 查询房间属性
     * @param roomID 房间ID
     * @param callback 回调
     */
    public void queryRoomAllAttributes(String roomID, IZIMCommonCallback callback){
        ZIMSDKManager.getInstance().queryRoomAllAttributes(roomID, new ZIMRoomAttributesQueriedCallback() {
            @Override
            public void onRoomAttributesQueried(String roomID, HashMap<String, String> roomAttributes, ZIMError errorInfo) {

                if(errorInfo.code.value() == 0){
                    parseAttributes(roomAttributes);
                }

                if(callback != null){
                    callback.onZIMCallback(errorInfo.code.value());
                }
            }
        });
    }

    /**
     * 首次查询房间属性，则直接设置信息
     * @param roomID 房间ID
     * @param callback 回调
     */
    public void firstQueryRoomAllAttributes(String roomID, IZIMCommonCallback callback){
        ZIMSDKManager.getInstance().queryRoomAllAttributes(roomID, new ZIMRoomAttributesQueriedCallback() {
            @Override
            public void onRoomAttributesQueried(String roomID, HashMap<String, String> roomAttributes, ZIMError errorInfo) {

                if(errorInfo.code.value() == 0){

                    if(roomAttributes == null){
                        return;
                    }

                    String infoStr = roomAttributes.get(INFO_TAG);
                    if(!TextUtils.isEmpty(infoStr)){
                        Info infoTemp = RoomAttributes.getGson().fromJson(infoStr,Info.class);
                        if(infoTemp != null){
                            mRoomAttributes.getInfo().setAuthor(infoTemp.getAuthor());
                        }
                    }

                    for(int i = 0; i < SEAT_LIST.length;i++){
                        Seat temp = RoomAttributes.getGson().fromJson(roomAttributes.get(SEAT_LIST[i]),Seat.class);

                        if(temp != null) {
                            L.e("ZIM","temp:" + i + " :  seat  = " + temp.getStatus());
                            if(temp.getStatus() == Seat.SeatStatus.INVITING.status){
                                temp.setStatus(Seat.SeatStatus.VACANT.status);
                            }

                            mRoomAttributes.setSeat(temp, i);


                            if (temp.getUser() != null && !TextUtils.isEmpty(temp.getUser().getId())) {
                                RoomManager.getInstance().applyUserService(
                                        services ->
                                        {
                                            User user = services.getUser(temp.getUser().getId());
                                            if (user == null) {
                                                user = new User();
                                                user.setId(temp.getUser().getId());
                                                user.setName(temp.getUser().getName());
                                                services.putUser(temp.getUser().getId(), user);
                                            }
                                            user.setUserIcon(temp.getUser().getUserIcon());
                                            user.setRole(temp.getUser().getRole());
                                            user.setIdx(temp.getUser().getIdx());

                                            user.setCanSpeak(temp.getUser().isCanSpeak());
                                            services.setUserStatus(temp.getUser().getId(), temp.getStatus());
                                        }
                                );

                            }

                            if(temp.getStatus() == Seat.SeatStatus.VACANT.status){
                                temp.setUser(null);
                            }


                        }

                    }

                }

                if(callback != null){
                    callback.onZIMCallback(errorInfo.code.value());
                }
            }
        });
    }


    /**
     * 解析属性并更新
     * @param roomAttributes 房间属性集合
     */
    private void parseAttributes(HashMap<String, String> roomAttributes){
        if(roomAttributes == null){
            return;
        }

        String infoStr = roomAttributes.get(INFO_TAG);

        HashMap<Integer, Seat> seatMap = new HashMap<>();
        for(int i = 0; i < SEAT_LIST.length;i++){
            Seat temp = RoomAttributes.getGson().fromJson(roomAttributes.get(SEAT_LIST[i]),Seat.class);
            seatMap.put(i,temp);
        }


        if(!TextUtils.isEmpty(infoStr)){
            Info infoTemp = RoomAttributes.getGson().fromJson(infoStr,Info.class);
            if(infoTemp != null){
                mRoomAttributes.getInfo().setAuthor(infoTemp.getAuthor());
            }
        }

        //重置历史用户信息
        resetHistoryUserInfo(seatMap);

        for(int i = 0; i < SEAT_LIST.length;i++){
            updateSeat(seatMap,i,mRoomAttributes.getSeat(i));
        }




    }

    /**
     * 更新麦位信息及用户信息
     */
    private void updateSeat(HashMap<Integer, Seat> seatMap,int position,Seat roomAttributesSeat){

            Seat temp = seatMap.get(position);
            if(temp != null){
                //房主无需修改自己的房间属性
                if (!RoomManager.getInstance().isHost() || !UserManager.getInstance().getUserID().equals(mRoomAttributes.getInfo().getAuthor())) {
                    roomAttributesSeat.setStatus(temp.getStatus());
                    roomAttributesSeat.setIdx(temp.getIdx());
                    if (temp.getUser() == null) {
                        roomAttributesSeat.setUser(null);
                    } else {
                        roomAttributesSeat.setUser(temp.getUser());
                    }
                }


                if(temp.getUser() != null &&  !TextUtils.isEmpty(temp.getUser().getId())){
                    RoomManager.getInstance().applyUserService(
                            services ->
                            {
                                User user = services.getUser(temp.getUser().getId());
                                if(user == null){
                                    user = new User();
                                    user.setId(temp.getUser().getId());
                                    user.setName(temp.getUser().getName());
                                    services.putUser(temp.getUser().getId(),user);
                                }
                                user.setUserIcon(temp.getUser().getUserIcon());
                                user.setRole(temp.getUser().getRole());
                                user.setIdx(temp.getUser().getIdx());
                                //如果用户id一致，且麦位状态为OCCUPIED，则说明用户自己目前属于麦上状态
                                if(UserManager.getInstance().getUserID().equals(temp.getUser().getId())
                                        && temp.getStatus() == Seat.SeatStatus.OCCUPIED.status){
                                    if(user.isCanSpeak() != temp.getUser().isCanSpeak()){
                                        if(!temp.getUser().isCanSpeak()){
                                            T.show(R.string.chathouse_you_silenced_by_the_anchor);
                                        }else{
                                            T.show(R.string.chathouse_you_remove_silenced_by_the_anchor);
                                        }
                                    }


                                }


                                user.setCanSpeak(temp.getUser().isCanSpeak());
                                services.setUserStatus(temp.getUser().getId(),temp.getStatus());
                            }
                    );

                    //如果用户id一致，且麦位状态为OCCUPIED，则说明用户自己目前属于麦上状态
//                    if(UserManager.getInstance().getUserID().equals(temp.getUser().getId())
//                            && temp.getStatus() == Seat.SeatStatus.OCCUPIED.status){
//
//
//
//                    }
                }
            }


    }

    /**
     * 重置历史用户信息
     * 如果之前麦位的用户不在最新的房间属性内，则进行重置
     */
    private void resetHistoryUserInfo(HashMap<Integer, Seat> seatMap){
        for(int i = 0; i < SEAT_LIST.length;i++){

            Seat lastSeat =  mRoomAttributes.getSeat(i);
            if(lastSeat.getUser() != null  && lastSeat.getUser().getId() != null && !isOccupySeat(seatMap,lastSeat)){
                RoomManager.getInstance().applyUserService(
                        services ->
                        {
                            User user = services.getUser(lastSeat.getUser().getId());
                            if(user != null){
                                L.i(TAG,"resetHistoryUserInfo:  seat = " + lastSeat.getIdx() + ": userID = " + user.getId());
                                user.setRole(User.UserRole.AUDIENCE.role);
                                user.setInviteTime(0);
                                user.setMicOn(true);
                                user.setIdx(-1);

                            }

                            services.setUserStatus(lastSeat.getUser().getId(),User.Status.VACANT.status);

                            //如果用户ID相同，则说明自己已经被下麦
                            if(UserManager.getInstance().getUserID().equals(lastSeat.getUser().getId())){
                                RoomManager.getInstance().applyInviteService(services1 -> services1.removeInvite(lastSeat.getIdx()));

                                if(lastSeat.getStatus() ==  Seat.SeatStatus.OCCUPIED.status && !UserManager.getInstance().getUserID().equals(mRoomAttributes.getInfo().getAuthor())){
                                    T.show(R.string.chathouse_you_have_been_anchor_down_seat);
                                }

                            }


                            if(RoomManager.getInstance().isHost()){
                                RoomManager.getInstance().applyInviteService(services1 -> services1.removeInvite(lastSeat.getIdx(),lastSeat.getUser().getId()));
                            }


                        }
                );
            }
        }
    }

    /**
     * 是否占据了麦位
     * 之前麦位占据的用户是否在这次的房间麦位中存在
     * @return 是否占据了麦位
     */
    private boolean isOccupySeat(HashMap<Integer, Seat> seatMap,Seat lastSeat){

        if(lastSeat != null && lastSeat.getUser() != null && lastSeat.getUser().getId() != null) {

            //如果上一个麦位在这次更新中未找到，则认为未更新
            if((lastSeat.getStatus() == Seat.SeatStatus.OCCUPIED.status || lastSeat.getStatus() == Seat.SeatStatus.INVITING.status) && seatMap.get(lastSeat.getIdx()) == null){
                return true;
            }


            for (int j = 0; j < SEAT_LIST.length; j++) {
                Seat seat = seatMap.get(j);
                if(seat != null && seat.getUser() != null){
                    if(lastSeat.getUser().getId().equals(seat.getUser().getId())
                    && (seat.getStatus() == Seat.SeatStatus.OCCUPIED.status || seat.getStatus() == Seat.SeatStatus.INVITING.status)){
                        return true;
                    }
                }
            }
        }
        return false;
    }

    /**
     * 通过userID获取麦位用户信息
     * @param userID 用户ID
     * @return 麦位
     */
    public Seat getSeatByUser(String userID){

        if(userID == null){
            return null;
        }

        for(int i = 0; i < SEAT_LIST.length;i++){
            Seat seat = mRoomAttributes.getSeat(i);
            if(seat != null && seat.getUser() != null){
                if(userID.equals(seat.getUser().getId())){
                    return seat;
                }
            }
        }
        return null;
    }


    /**
     * 获取房间属性
     * @return 房间属性
     */
    public RoomAttributes getRoomAttributes() {
        return mRoomAttributes;
    }


    public boolean isCanUpdateRoomAttributes() {
        return mCanUpdateRoomAttributes;
    }

    public void setCanUpdateRoomAttributes(boolean canUpdateRoomAttributes) {

        this.mCanUpdateRoomAttributes = canUpdateRoomAttributes;
    }
}
