package im.zego.chathouse.room.service;

import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.CONNECTION_STATE;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import im.zego.chathouse.sdk.constants.ConnectionState;
import im.zego.chathouse.sdk.rtc.RTCEventHandlerWrap;
import im.zego.chathouse.sdk.rtc.RTCSDKManager;
import im.zego.chathouse.sdk.rtc.callbacks.IConnectionStateChangeListener;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.zim.ZIM;
import im.zego.zim.enums.ZIMConnectionEvent;
import im.zego.zim.enums.ZIMConnectionState;
import im.zego.zim.enums.ZIMRoomEvent;
import im.zego.zim.enums.ZIMRoomState;

/**
 * 房间SDK连接服务
 */
public class RoomConnectService {

    private final String mRoomID;

    private ZIMRoomState mZimConnectionState = ZIMRoomState.CONNECTED;
    private ConnectionState mRoomState = ConnectionState.CONNECTED;
    private ConnectionState mConnectionState = ConnectionState.CONNECTED;

    private final List<IConnectionStateChangeListener> mConnectionStateChangeList = new ArrayList<>();

    private final ZIMEventHandlerWrap zimEventHandlerWrap;

    private final RTCEventHandlerWrap rtcEventHandlerWrap;

    public RoomConnectService(String roomID){
        mRoomID = roomID;

        zimEventHandlerWrap = new ZIMEventHandlerWrap(mRoomID,CONNECTION_STATE) {
            @Override
            public void onConnectionStateChanged(ZIM zim, ZIMConnectionState state, ZIMConnectionEvent event, JSONObject extendedData) {
                if(state == ZIMConnectionState.DISCONNECTED){
                    mZimConnectionState = ZIMRoomState.DISCONNECTED;
                    onStateChange();
                }

            }

            @Override
            public void onRoomStateChanged(ZIM zim, ZIMRoomState state, ZIMRoomEvent event, JSONObject extendedData, String roomID) {
                super.onRoomStateChanged(zim, state, event, extendedData, roomID);
                //连接中和正在连接使用房间状态变化，更好确认房间连接时机
                if(state == ZIMRoomState.CONNECTED || state == ZIMRoomState.CONNECTING ){
                    mZimConnectionState = state;
                    onStateChange();
                }
            }
        };

        rtcEventHandlerWrap = new RTCEventHandlerWrap(mRoomID,CONNECTION_STATE) {
            @Override
            public void onConnectionState(ConnectionState state) {
                mRoomState = state;
                onStateChange();
            }
        };
    }



    /**
     * 初始化
     * 使用时机：在ZIM SDK 初始化后 且 RTC 初始化后调用
     */
    public void init() {
        mZimConnectionState = ZIMRoomState.CONNECTED;
        mRoomState = ConnectionState.CONNECTED;
        mConnectionState = ConnectionState.CONNECTED;

        ZIMSDKManager.getInstance().getDispatchService().registerZIMEventHandlerCallbacks(zimEventHandlerWrap);

        RTCSDKManager.getInstance().getDispatchService().registerRTCEventHandlerCallbacks(rtcEventHandlerWrap);
    }

    /**
     * 状态变化时，通知更新
     */
    private void onStateChange(){

        //两个SDK 都连接成功则认为网络连接成功
        if(mRoomState == ConnectionState.CONNECTED && mZimConnectionState == ZIMRoomState.CONNECTED){

            if(mConnectionState != ConnectionState.CONNECTED){
                mConnectionState = ConnectionState.CONNECTED;
                notifyStateChange();
            }
            //其中一个SDK彻底断开连接，则认为断开
        }else if(mRoomState == ConnectionState.DISCONNECTED || mZimConnectionState == ZIMRoomState.DISCONNECTED){

            if(mConnectionState != ConnectionState.DISCONNECTED){
                mConnectionState = ConnectionState.DISCONNECTED;
                notifyStateChange();
            }

        }else {

            if(mConnectionState != ConnectionState.CONNECTING){
                mConnectionState = ConnectionState.CONNECTING;
                notifyStateChange();
            }
        }

    }

    /**
     * 通知状态更新
     */
    private void notifyStateChange(){
        for(IConnectionStateChangeListener listener : mConnectionStateChangeList){
            if(listener != null){
                listener.onConnectionState(mConnectionState);
            }
        }
    }

    public ConnectionState getConnectionState(){
        return mConnectionState;
    }

    public void addConnectionStateChange(IConnectionStateChangeListener connectionStateChangeListener){
        mConnectionStateChangeList.add(connectionStateChangeListener);
    }

    public void removeConnectionStateChange(IConnectionStateChangeListener connectionStateChangeListener){
        mConnectionStateChangeList.remove(connectionStateChangeListener);
    }

    /**
     * 反初始化
     * 使用时机：在ZIM SDK 反初始化前 且 RTC 反初始化前调用
     */
    public void unInit() {
        mConnectionStateChangeList.clear();
        ZIMSDKManager.getInstance().getDispatchService().unregisterZIMEventHandlerCallbacks(zimEventHandlerWrap);
        RTCSDKManager.getInstance().getDispatchService().unregisterRTCEventHandlerCallbacks(rtcEventHandlerWrap);
    }

}
