package im.zego.chathouse.room.service;

import static im.zego.chathouse.Constants.MIX_STREAM_SUFFIX;
import static im.zego.chathouse.model.RoomAttributes.SEAT_LIST;
import static im.zego.chathouse.sdk.rtc.RTCEventHandlerWrap.DATA_PRIORITY;
import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.DATA_PROCESSING_COMPLETED_PRIORITY;
import static im.zego.zegoexpress.constants.ZegoUpdateType.ADD;
import static im.zego.zegoexpress.constants.ZegoUpdateType.DELETE;


import android.text.TextUtils;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.RoomAttributes;
import im.zego.chathouse.model.Seat;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.sdk.rtc.RTCEventHandlerWrap;
import im.zego.chathouse.sdk.rtc.RTCSDKManager;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.enjoycommon.log.L;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoStream;
import im.zego.zim.ZIM;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;

public class RoomStreamService {

    private static final String TAG = "RoomStreamService";

    private final String mRoomID;
    private final ZIMEventHandlerWrap mZimEventHandlerWrap;
    private final RTCEventHandlerWrap mRTCEventHandlerWrap;

    /**
     * 房间内除了混流外的所有的流（userID : streamID）
     */
    private final Map<String,String> streamMap = new HashMap<>();

    /**
     * 在在推流或者拉流的流列表
     */
    private final Set<String> streamSet = new HashSet<>();

    /**
     * 上次混流的集合（麦位：流ID）
     */
    private final Map<Integer, String> lastMixStreamMap = new HashMap<>();

    public RoomStreamService(String roomID){
        mRoomID = roomID;
        mZimEventHandlerWrap = new ZIMEventHandlerWrap(mRoomID,DATA_PROCESSING_COMPLETED_PRIORITY) {
            @Override
            public void onRoomAttributesUpdated(ZIM zim, ZIMRoomAttributesUpdateInfo info, String roomID) {
                super.onRoomAttributesUpdated(zim, info, roomID);

                updateStream();
            }
        };

        mRTCEventHandlerWrap = new RTCEventHandlerWrap(mRoomID,DATA_PRIORITY) {
            @Override
            public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData) {
                super.onRoomStreamUpdate(roomID, updateType, streamList, extendedData);

                for (ZegoStream stream : streamList) {
                    L.i(TAG,"onRoomStreamUpdate:" + " : updateType : " + updateType + " : streamID : " + stream.streamID);
                    //记录麦上的流
                    if (updateType == ADD) {
                        if (!stream.streamID.contains(MIX_STREAM_SUFFIX)) {
                            streamMap.put(stream.user.userID, stream.streamID);
                        }
                    } else if (updateType == DELETE) {
                        if (!stream.streamID.contains(MIX_STREAM_SUFFIX)) {
                            streamMap.remove(stream.user.userID);
                        }
                    }
                }

                updateStream();
            }

            @Override
            public void onRemoteMicStateUpdate(String streamID, boolean open) {
                super.onRemoteMicStateUpdate(streamID, open);

                if(!TextUtils.isEmpty(streamID)) {
                    for (Map.Entry<String, String> entry : streamMap.entrySet()) {
                        if (streamID.equals(entry.getValue())){

                            RoomManager.getInstance().applyUserService(services -> {
                                services.setUserMic(entry.getKey(),open);
                                RoomManager.getInstance().applyRoomAttributesService(services2 -> services2.setMicOnOff(entry.getKey(),open));

                            });


                        }
                    }
                }
            }



        };


    }

    /**
     * 更新流服务
     */
    public void updateStream(){

        RoomManager.getInstance().applyRoomAttributesService(services -> {

            Map<Integer,String> seatMap = new HashMap<>();

            RoomAttributes roomAttributes = services.getRoomAttributes();
            for(int i = 0; i < SEAT_LIST.length;i++){
                Seat seat = roomAttributes.getSeat(i);

                if(seat != null && seat.getStatus() == Seat.SeatStatus.OCCUPIED.status &&
                        seat.getUser() != null && seat.getUser().getId() != null){
                    seatMap.put(i,seat.getUser().getId());
                }

            }

            //如果在麦上就拉单流，否则拉混流
            if(isOccupiedSeat(seatMap)){
                startSingleStreamTask(seatMap);
            }else {
                startMixStreamTask();
            }

        });
    }

    /**
     * 自己是否在麦上
     *
     * @param seatMap 麦位集合（麦位 ： 用户ID）
     * @return true 在麦上
     */
    public boolean isOccupiedSeat(Map<Integer,String> seatMap ){
        for(String uid:seatMap.values()){
            if(UserManager.getInstance().getUserID().equals(uid)){
                return true;
            }
        }
        return false;
    }

    /**
     * 麦下用户拉去混流
     */
    public void startMixStreamTask(){

        //如果不是混流，则停止拉单流
        if(streamSet.size() != 0 && (streamSet.size() != 1 || !streamSet.contains(getMixStreamID()))){
            for(String stream:streamSet){
                if(getMainStreamId().equals(stream)){
                    RTCSDKManager.getInstance().stopPublishingStream();
                }else {
                    RTCSDKManager.getInstance().stopPlayingStream(stream);
                }
            }
            streamSet.clear();
        }


        if(!streamSet.contains(getMixStreamID())) {
            RTCSDKManager.getInstance().startPlayingStream(getMixStreamID());
            streamSet.add(getMixStreamID());

        }
    }

    /**
     * 麦上用户去拉单流
     *
     * @param seatMap 麦位map (麦位：用户ID)
     */
    public void startSingleStreamTask(Map<Integer,String> seatMap){
        //如果有正在推拉的流不在麦位上，则停止推拉
        String[] strings = streamSet.toArray(new String[0]);

        for(String stream :strings){

            if(!TextUtils.isEmpty(stream) ){

                if(!isSeatStream(stream,seatMap)) {
                    //如果是自己
                    if (stream.equals(getMainStreamId())) {

                        RTCSDKManager.getInstance().stopPublishingStream();
                    } else {

                        RTCSDKManager.getInstance().stopPlayingStream(stream);
                    }
                    streamSet.remove(stream);
                }
            }
        }

        //拉取所有在麦位上的流

        Collection<String> userIDList = seatMap.values();
        for(String userId:userIDList){


            if(UserManager.getInstance().getUserID().equals(userId)){

                if(!streamSet.contains(getMainStreamId())) {
                    RTCSDKManager.getInstance().startPublishingStream(getMainStreamId());
                    streamSet.add(getMainStreamId());
                }
            }else {
                String streamID = streamMap.get(userId);
                if (!TextUtils.isEmpty(streamID)) {

                    if(!streamSet.contains(streamID)) {
                        RTCSDKManager.getInstance().startPlayingStream(streamID);
                        streamSet.add(streamID);

                    }
                }
            }
        }

        if(RoomManager.getInstance().isHost()){
            //手动混流
            updateMixStream();
            //自动混流
//            RTCSDKManager.getInstance().autoMixerStream();
        }

    }

    /**
     * 通过流ID,获取用户ID
     *
     * @param streamID 流ID
     * @return 用户ID
     */
    public String getUserIDByStreamID(String streamID){

        if(TextUtils.isEmpty(streamID)){
            return null;
        }

        if(streamID.equals(getMainStreamId())){
            return UserManager.getInstance().getUserID();
        }

        for(Map.Entry<String, String> entry: streamMap.entrySet()) {

            if(streamID.equals(entry.getValue())){
                return entry.getKey();
            }
        }
        return null;
    }



    /**
     * 此流是否是麦位上的流
     *
     * @param stream 流ID
     * @param seatMap 麦位信息
     * @return true 是麦位上的流
     */
    public boolean isSeatStream(String stream,Map<Integer,String> seatMap){
        Collection<String> userIDList = seatMap.values();
        for(String userId:userIDList){


            if(!TextUtils.isEmpty(userId)){

                if(userId.equals(UserManager.getInstance().getUserID())
                    && getMainStreamId().equals(stream)){
                    return true;
                }

                String streamID = streamMap.get(userId);
                if(stream.equals(streamID)){
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 更新混流
     */
    public void updateMixStream(){
        RoomManager.getInstance().applyRoomAttributesService(services -> {

            //混流集合（麦位 ：流ID）
            Map<Integer, String> mixStreamMap = new HashMap<>();
            //麦位集合（麦位 ：用户ID）
            Map<Integer, String> seatMap = new HashMap<>();

            RoomAttributes roomAttributes = services.getRoomAttributes();
            for (int i = 0; i < SEAT_LIST.length; i++) {
                Seat seat = roomAttributes.getSeat(i);

                if (seat != null && seat.getStatus() == Seat.SeatStatus.OCCUPIED.status &&
                        seat.getUser() != null && seat.getUser().getId() != null) {
                    seatMap.put(i, seat.getUser().getId());
                }
            }

            for(Map.Entry<Integer, String> entry: seatMap.entrySet()){

                //如果和自己的uid相同，则直接添加,不同，则查找房间内是否存在流
                if(UserManager.getInstance().getUserID().equals(entry.getValue())){
                    mixStreamMap.put(entry.getKey(),getMainStreamId());
                }else if(!TextUtils.isEmpty(streamMap.get(entry.getValue()))){
                    mixStreamMap.put(entry.getKey(),streamMap.get(entry.getValue()));
                }

            }

            if(!compareMap(lastMixStreamMap,mixStreamMap)) {
                RTCSDKManager.getInstance().updateMixStream(mixStreamMap);
            }
            lastMixStreamMap.clear();
            lastMixStreamMap.putAll(mixStreamMap);

        });
    }




    /**
     * 比较两个map是否相同
     *
     * @param lastMap 之前流的map （麦位 ：流ID）
     * @param map 现在流的map （麦位 ：流ID）
     * @return 是否相同
     */
    public boolean compareMap(Map<Integer, String> lastMap,Map<Integer, String> map){

        if(lastMap.size() != map.size()){
            return false;
        }

        for (Map.Entry<Integer, String> entry : map.entrySet()) {
            String value = entry.getValue() == null ? "" : entry.getValue();

            if (!value.equals(lastMap.get(entry.getKey()))) {
                return false;
            }

        }

        return true;
    }


    /**
     * 获取自己推流的ID
     *
     * @return 流ID
     */
    private String getMainStreamId(){
        return RTCSDKManager.getInstance().getMainStreamId();
    }

    /**
     * 获取混流ID
     *
     * @return 流ID
     */
    private String getMixStreamID(){
        return RTCSDKManager.getInstance().getMixStreamID();
    }

    public void init(){
        ZIMSDKManager.getInstance().getDispatchService().registerZIMEventHandlerCallbacks(mZimEventHandlerWrap);
        RTCSDKManager.getInstance().getDispatchService().registerRTCEventHandlerCallbacks(mRTCEventHandlerWrap);
    }

    public void unInit(){
        ZIMSDKManager.getInstance().getDispatchService().unregisterZIMEventHandlerCallbacks(mZimEventHandlerWrap);
        RTCSDKManager.getInstance().getDispatchService().unregisterRTCEventHandlerCallbacks(mRTCEventHandlerWrap);
        streamMap.clear();
        streamSet.clear();
    }

}
