package im.zego.chathouse.roomlist.view;

import android.content.Context;
import android.graphics.Rect;

import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

/**
 * 房间列表分割线
 */
public class RoomListDecoration extends RecyclerView.ItemDecoration {

    private final Context mContext;
    private final int mFirstRowTopMargin;
    private final int mGridIntervalDistance;
    private final int mMarginHorizontal;


    /**
     *
     * @param context 上下文
     * @param firstRowTopMargin 首行距离顶部的高度
     * @param gridIntervalDistance 列表间间隔距离
     * @param marginHorizontal 列表左右外边距
     */
    public RoomListDecoration(Context context,float firstRowTopMargin,float gridIntervalDistance,float marginHorizontal) {
        this.mContext = context;
        mFirstRowTopMargin = dip2px(firstRowTopMargin);
        mGridIntervalDistance = dip2px(gridIntervalDistance);
        mMarginHorizontal = dip2px(marginHorizontal);
    }

    /**
     * 获取总列数
     * @param parent 列表视图
     * @return 总列数
     */
    private int getSpanCount(RecyclerView parent) {
        int spanCount = -1;
        RecyclerView.LayoutManager layoutManager = parent.getLayoutManager();
        if (layoutManager instanceof GridLayoutManager) {
            spanCount = ((GridLayoutManager) layoutManager).getSpanCount();
        }
        return spanCount;
    }

    /**
     * 首行判断
     * @param parent 列表视图
     * @return true 首行
     */
    private boolean isFirstRow(RecyclerView parent, int pos, int spanCount,
                               int childCount) {
        RecyclerView.LayoutManager layoutManager = parent.getLayoutManager();
        if (layoutManager instanceof GridLayoutManager) {
            return (pos / spanCount) == 0;
        }
        return false;
    }


    /**
     * 首列判断
     * @return true 首列
     */
    private boolean isFirstColumn(RecyclerView parent, int pos, int spanCount,
                                  int childCount) {
        RecyclerView.LayoutManager layoutManager = parent.getLayoutManager();
        if (layoutManager instanceof GridLayoutManager) {
            return (pos + 1) % spanCount == 1;
        }
        return false;
    }

    /**
     * 判断是最后一列
     * @return true 最后一列
     */
    private boolean isLastColumn(RecyclerView parent, int pos, int spanCount,
                                 int childCount) {
        RecyclerView.LayoutManager layoutManager = parent.getLayoutManager();
        if (layoutManager instanceof GridLayoutManager) {
            return (pos + 1) % spanCount == 0;
        }
        return false;
    }

    /**
     * 判断是最后一行
     * @return 最后一行
     */
    private boolean isLastRow(RecyclerView parent, int pos, int spanCount,
                              int childCount) {
        RecyclerView.LayoutManager layoutManager = parent.getLayoutManager();
        if (layoutManager instanceof GridLayoutManager) {
            int ranger = childCount % spanCount;
            if (ranger == 0) {
                ranger = spanCount;
            }
            childCount = childCount - ranger;
            return pos >= childCount;
        }
        return false;
    }

    /**
     * 设置每个item的偏移量，从而展示分割线
     */
    @Override
    public void getItemOffsets(Rect outRect, int itemPosition,
                               RecyclerView parent) {
        int spanCount = getSpanCount(parent);
        int childCount = parent.getAdapter().getItemCount();
        int marginLeft = 0;
        int marginRight = mGridIntervalDistance;
        int marginBottom = mGridIntervalDistance;


        //如果总数为1，表示是展示空视图
        if(childCount == 1){
            outRect.set(0, 0, 0, 0);
            return;
        }

        // 如果是第一列，则计算左边的外边距
        if(isFirstColumn(parent, itemPosition, spanCount, childCount)){
            marginLeft = mMarginHorizontal;
        }

        // 如果是最后一行，则不需要绘制底部
        if (isLastRow(parent, itemPosition, spanCount, childCount))
        {
            marginBottom = 0;
        }

        // 如果是最后一列，则不需要绘制右边，只计算右边的外边距
        if (isLastColumn(parent, itemPosition, spanCount, childCount))
        {
            marginRight = mMarginHorizontal;
        }

        if(isFirstRow(parent,itemPosition,spanCount,childCount)){
            outRect.set(marginLeft, mFirstRowTopMargin, marginRight, marginBottom);
        }else {
            outRect.set(marginLeft, 0, marginRight, marginBottom);
        }


    }

    /**
     * 根据手机的分辨率从 dp 的单位 转成为 px(像素)
     */
    public int dip2px(float dpValue) {
        final float scale = mContext.getResources().getDisplayMetrics().density;
        return (int) (dpValue * scale + 0.5f);
    }


}
