package im.zego.enjoycommon.util;

import android.os.Handler;
import android.os.Message;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleObserver;
import androidx.lifecycle.OnLifecycleEvent;

import java.lang.ref.SoftReference;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * 防止内存泄漏的Handler
 *  * 1. 采用软引用
 *  * 2. 自动销毁，移除message
 *  * 3. 查询是否存在指定消息
 * @param <T>
 */
public class SafetyHandler<T> extends Handler implements LifecycleObserver {

    private SoftReference<Task<T>> softReference;
    private Lifecycle lifecycle;
    private AtomicBoolean destroy=new AtomicBoolean(false);

    /**
     * 在FragmentActivity中使用
     * @param activity
     * @param <T>
     * @return
     */
    public static <T> SafetyHandler<T> with(FragmentActivity activity) {
        SafetyHandler<T> handler = new SafetyHandler<>();
        handler.lifecycle = activity.getLifecycle();
        handler.lifecycle.addObserver(handler);
        return handler;
    }

    /**
     * 在Fragment中使用
     * @param fragment
     * @param <T>
     * @return
     */
    public static <T> SafetyHandler<T> with(Fragment fragment) {
        SafetyHandler<T> handler = new SafetyHandler<>();
        handler. lifecycle = fragment.getLifecycle();
        handler. lifecycle.addObserver(handler);
        return handler;
    }

    public SafetyHandler<T> bind(T bindObject, HandlerListener<T> listener) {
        this.softReference = new SoftReference<>(new Task<>(bindObject, listener));
        return this;
    }


    @Override
    public boolean sendMessageAtTime(Message msg, long uptimeMillis) {
        if (destroy.get()){
            return false ;
        }
        boolean result = super.sendMessageAtTime(msg, uptimeMillis);
        return result;
    }


    @Override
    public void handleMessage(Message msg) {
        Task<T> task = getReference();
        if (task != null && task.listener != null) {
            task.listener.handMessage(task.bindObject, msg);
        }
    }

    protected Task<T> getReference() {
        return softReference == null ? null : softReference.get();
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    public void onDestroy() {

        releaseTask();
        releaseReference();
        this.destroy.compareAndSet(false,true);
        if (lifecycle != null) {
            lifecycle.removeObserver(this);
            lifecycle = null;
        }
    }

    private static class Task<T> {
        private T bindObject;
        private HandlerListener<T> listener;

        public Task(T bind, HandlerListener<T> listener) {
            this.bindObject = bind;
            this.listener = listener;
        }
    }

    public interface HandlerListener<T> {
        void handMessage(T bind, Message message);
    }

    /**
     * 释放引用
     */
    private void releaseReference() {
        if (getReference() != null) {
            softReference.clear();
            softReference = null;
        }
    }

    /**
     * 释放Handler处理的任务
     */
    private void releaseTask() {
        this.removeCallbacksAndMessages(null);
    }

}
